import json
from typing import Any, Type, Optional
from pydantic import BaseModel, Field, create_model
from enum import Enum

def json_schema_to_base_model(schema) :
    type_mapping = {
        "string": str,
        "integer": int,
        "number": float,
        "boolean": bool,
        "array": list,
        "object": dict,
    }

    properties = schema.get("properties", {})
    required_fields = schema.get("required", [])
    model_fields = {}

    for field_name, field_props in properties.items():
        json_type = field_props.get("type", "string")
        enum_values = field_props.get("enum")

        if enum_values:
            enum_name = f"{field_name.capitalize()}Enum"
            field_type = Enum(enum_name, {v: v for v in enum_values})
        else:
            field_type = type_mapping.get(json_type, Any)

        default_value = field_props.get("default", ...)
        nullable = field_props.get("nullable", False)
        description = field_props.get("title", "")

        if nullable:
            field_type = Optional[field_type]

        if field_name not in required_fields:
            default_value = field_props.get("default", None)

        model_fields[field_name] = (field_type, Field(default_value, description=description))

    return create_model(schema.get("title", "DynamicModel"), **model_fields)


path_json=r"C:\Rubtsov\Python\AxiomaPlugins\ru_axioma_gis_stamp_to_report\ui\tools\shema_cls_params.json"
path_json=r"C:\Rubtsov\Python\AxiomaPlugins\ru_axioma_gis_stamp_to_report\ui\tools\shema_cls_params_default.json"
with open(path_json, encoding='utf-8') as fh:
    data = json.load(fh)
cls_params = json_schema_to_base_model(data )
cls_params.developer="Сидоров С.C"
cls_params.contoller="Сидоров С.C"
cls_params.chif_engener="Сидоров С.C"
jkl=0
print(cls_params.schema_json(indent=2))