from PySide2.QtGui import QTransform
from axipy import Pnt, Geometry, GeometryReportItem

def correctMifText(mif_text):
    ind_start = mif_text.rfind('"')
    sub_bound = mif_text[ind_start + 1:len(mif_text)].strip()
    xy_bound = sub_bound.split(' ')
    xmin = float(xy_bound[0])
    ymin = float(xy_bound[1])
    xmax = float(xy_bound[2])
    ymax = float(xy_bound[3])
    dy = ymax - ymin
    pt1 = Pnt(xmin, ymin - dy)

    pt2 = Pnt(xmax, ymax - dy)
    out_mif = mif_text[0:ind_start + 1] + " " + str(pt1.x) + " " + str(pt1.y) + " " + str(pt2.x) + " " + str(pt2.y)
    return Geometry.from_mif(out_mif)
def addItemToReport(report,geo_obj,style):
    geomItem = GeometryReportItem()
    geomItem.geometry = geo_obj
    geomItem.style = style
    report.items.add(geomItem)
    return
class AxiTransform:
    def __init__(self,dx,dy,angle,scale,pt_rotate=None):
        self.__dx=dx
        self.__dy=dy
        self.__angle=angle
        self.__scale=scale
        self.__pt_rotate=pt_rotate
        if pt_rotate is None:
            self.__pt_rotate=Pnt(0,0)
        self.__tr_scale=QTransform.fromScale(scale,-scale)

    def transform(self,geo:Geometry):

        #new_geo=geo.scale(self.__scale,self.__scale)
        new_geo = geo.affine_transform(self.__tr_scale)

        geo_rotate=new_geo.rotate(self.__pt_rotate,self.__angle)
        geo_offset=geo_rotate.shift(self.__dx,self.__dy)
        return geo_offset
