import os
from pathlib import Path

import axipy
from PySide2.QtCore import QFile
from PySide2.QtGui import QPageLayout
from PySide2.QtUiTools import QUiLoader
from PySide2.QtWidgets import QWidget
from axipy import UnitValue, LinearUnits, view_manager, data_manager, show_message, DlgIcon, Rectangle

from ru_axioma_gis_stamp_to_report.ui.tools.Observers import factoryObserverSelectPolygonsNew, \
    factoryObserverSelectPolygon
from ru_axioma_gis_stamp_to_report.ui.tools.helper_tab import pathTabIsOpen, removeTable

from ru_axioma_gis_stamp_to_report.ui.tools.pages_report.helper_bounds import CalculatorListFromBound


class CreateListsTab(QWidget):
    def __init__(self,parent_object,parent=None):
        #super().__init__(parent)
        QWidget.__init__(self, parent)
        self.__parent_object=parent_object
        self.__observer = factoryObserverSelectPolygonsNew("SelectPolygons")
        self.__observer.changed.connect(self.__select_polygon)
        self.__ui=self.load_ui("form_create_list_3.ui",parent)
        self.__ui.tb_select_out_tab.clicked.connect(self.__select_out_tab)
        self.__ui.cmb_scale.currentIndexChanged.connect(self.__change_scale)
        self.__init_scale()
        self.__setInfoPageSize()
        self.__ui.pb_run.clicked.connect(self.__run)
        observer = factoryObserverSelectPolygon()
        self.__ui.chb_select_polygon.setEnabled(observer.value)
        observer.changed.connect(self.__observerChange)
    def load_ui(self,name_resource,parent):
        loader = QUiLoader()
        path = os.path.join(os.path.dirname(__file__),name_resource)
        ui_file = QFile(path)
        ui_file.open(QFile.ReadOnly)
        ui  = loader.load(ui_file,parent)
        ui_file.close()
        return ui
    def __init_scale(self):
        scales=['100','250','500','1000','2000','5000','10000','25000']
        self.__ui.cmb_scale.addItems(scales)
    def __select_polygon(self):
        self.__ui.chb_select_polygon.setEnabled(self.__observer.value)
    def __setInfoPageSize(self):
        page_info=self.__parent_object.getCurentSizeList()
        #page_size,orientation =self.__parent_object.getCurentSizeList()
        text_inf=page_info.page_size.name.decode('utf-8')
        text_orient="Портрет"
        if page_info.orientation==QPageLayout.Landscape:
            text_orient = "Альбом"
        text_inf=text_inf+" "+text_orient
        self.__ui.label_format_list.setText(text_inf)
    def update_page_info(self):
        self.__setInfoPageSize()
    def __select_out_tab(self):
        title = 'Сохранить таблицу разбивки на листы'
        filter = 'MapInfo Tab (*.tab)'
        curent_scale=self.__ui.cmb_scale.currentText()
        file_name="list_"+curent_scale+".tab"
        selected_filter = 'MapInfo Tab (*.tab)'
        path_out=axipy.save_file_dialog(filter,title,None,file_name,selected_filter)
        if path_out is None:
            return
        str_path=str(path_out)
        self.__ui.ln_path_out_tab.setText(str_path)
        self.__ui.ln_path_out_tab.setCursorPosition(0)
        self.__ui.pb_run.setEnabled(True)
    def __change_scale(self):
        if len(self.__ui.ln_path_out_tab.text().replace(' ',''))==0:
            return
        path_out=Path(self.__ui.ln_path_out_tab.text())
        name="list_"+self.__ui.cmb_scale.currentText()+".tab"
        out_path=str(path_out.parent.joinpath(name))
        self.__ui.ln_path_out_tab.setText(out_path)
        self.__ui.ln_path_out_tab.setCursorPosition(0)
    def __run(self):
        str_path_out=self.__ui.ln_path_out_tab.text()
        tab_open=pathTabIsOpen(str_path_out)
        name_tab=Path(str_path_out).name
        if tab_open is not None:
            name_tab = tab_open.name
            data_manager.remove(tab_open)
            tab_open.close()
        isOkDeleteTab=removeTable(str_path_out)
        if not isOkDeleteTab:
            show_message('Таблица '+name_tab+" закрыта , но удалить не удалось. Задайте другое имя таблицы ", 'Заголовок', icon=DlgIcon.INFORMATION)
            return
        bound_page=None
        if self.__ui.ch_overStamp.isChecked():
            bound_page=self.__parent_object.getBoundFull()
        else:
            if self.__ui.rb_stamp_8.isChecked():
                bound_page = self.__parent_object.getBoundMapStampBig()
            else:
                bound_page = self.__parent_object.getBoundMapStampStandart()
        self.__ui.pb_cancel.setEnabled(True)
        self.__ui.pb_run.setEnabled(False)
        scale=int(self.__ui.cmb_scale.currentText())
        scale_unit=UnitValue(scale,LinearUnits.cm)
        source_bound=None
        if self.__ui.chb_select_polygon.isChecked():
            tab_sel = data_manager.selection

            geo_selection=list(tab_sel.items())[0].geometry
            if geo_selection.coordsystem!=view_manager.active.coordsystem:
                geo_selection=geo_selection.reproject(view_manager.active.coordsystem)
            rect=geo_selection.bounds
            source_bound=Rectangle(rect,cs=view_manager.active.coordsystem)
        self.__cls_run=CalculatorListFromBound(source_bound,view_manager.active,UnitValue(bound_page.width,LinearUnits.cm),UnitValue(bound_page.height,LinearUnits.cm),scale_unit,str_path_out)
        self.__cls_run.change_progress.connect(self.__update_progess)
        self.__cls_run.change_progress_bar_format.connect(self.__update_progess_format)

        def run_process(t: axipy.Task):
            self.__cls_run.run(True)

        self.__run_task = axipy.Task(run_process)
        self.__run_task.finished.connect(self.__finish_task)
        self.__run_task.start()

    def __finish_task(self):
        self.__cls_run.change_progress.disconnect(self.__update_progess)
        self.__cls_run.change_progress_bar_format.disconnect(self.__update_progess_format)
        self.__ui.pb_run.setEnabled(True)
        self.__ui.pb_cancel.setEnabled(False)


        self.__cls_run=None
        jkl=0
    @property
    def widget(self):
        return self.__ui

    def __update_progess(self, value):
        # print(value)
        self.__ui.progressBar.setValue(value)
    def __update_progess_format(self,header):
        self.__ui.progressBar.setFormat(header +  " .. %p%")
    def __observerChange(self,value):
        self.__ui.chb_select_polygon.setEnabled(value)

