import os
import datetime

import axipy
from PySide2.QtCore import QFile
from PySide2.QtUiTools import QUiLoader
from PySide2.QtWidgets import QWidget, QHBoxLayout, QLabel, QLineEdit, QListWidgetItem, QVBoxLayout, QTextEdit

from ru_axioma_gis_stamp_to_report.ui.tools.helper_params_stamp import factoryParamsStamp, factoryParamsDateCreate, \
    saveObjectToJson
from ru_axioma_gis_stamp_to_report.ui.tools.helper_tab import getLastModifiFiles
class WidgetParamHeader(QWidget):
    def __init__(self,name_object,object_params,parent=None):
        QWidget.__init__(self,parent)
        self.name_object = name_object
        self.__object_params=object_params
        layout = QVBoxLayout()
        self.setLayout(layout)
        name_params=QLabel()
        name_params.setStyleSheet("font-weight: bold")
        name = str(object_params.title)
        name_params.setText(name)
        layout.addWidget(name_params)
        self.__params_value=QTextEdit()
        self.__params_value.setMinimumHeight(20)
        self.__params_value.setMaximumHeight(60)
        value=str(object_params.value)

        self.__params_value.setText(value)
        self.__params_value.adjustSize()

        layout.addWidget(self.__params_value)
    def getUpdatedObject(self):
        self.__object_params.value._value=self.__params_value.toPlainText()
        return self.__object_params

class WidgetParamDate(QWidget):
    def __init__(self,name_object,object_params,date_def_width=60,parent=None):
        QWidget.__init__(self,parent)
        self.__object_params=object_params
        self.name_object=name_object
        layout = QHBoxLayout()
        self.setLayout(layout)
        name_params=QLabel()
        name_params.setStyleSheet("font-weight: bold")
        name=str(object_params.title)
        name_params.setText(name)
        layout.addWidget(name_params)
        value=str(object_params.value)
        self.__params_value=QLineEdit()
        self.__params_value.setText(value)
        layout.addWidget(self.__params_value)
        self.__params_value_date=None
        if type(object_params).__name__=='objSimpleNoData':
            return
        name_params_date = QLabel()
        name_params_date.setText("Дата")
        layout.addWidget(name_params_date)
        self.__params_value_date = QLineEdit()
        value_date=""
        if object_params.date_create is None:
            current_time = datetime.datetime.now()
            month=current_time.month
            year=current_time.year
            value_date=str(month)+"."+str(year)
        else:
            value_date=str(object_params.date_create.value)
            date_def_width=int(str(object_params.date_create.width_max))
        self.__params_value_date.setText(value_date)
        self.__params_value_date.adjustSize()
        self.__params_value_date.setMaximumWidth(date_def_width)
        layout.addWidget(self.__params_value_date)

    def getUpdatedObject(self):
        self.__object_params.value._value=self.__params_value.text()
        if self.__params_value_date is None:
            return self.__object_params
        if self.__object_params.date_create is None:
            self.__object_params.date_create=factoryParamsDateCreate(self.__params_value_date.text())
        else:
            self.__object_params.date_create.value._value=self.__params_value_date.text()
        return self.__object_params


class WidgetParamsStamp(QWidget):
    def __init__(self,parent_object,parent=None):
        QWidget.__init__(self, parent)
        self.__parent_object = parent_object
        self.__ui=self.load_ui("from_params_stamp.ui",parent)
        self.__initPramsStampClass()
        self.__ui.pb_apply.clicked.connect(self.__apply_cls_value)
        self.__ui.pb_save.clicked.connect(self.__save)
        self.__ui.pb_load.clicked.connect(self.__load)
    def load_ui(self,name_resource,parent):
        loader = QUiLoader()
        path = os.path.join(os.path.dirname(__file__),name_resource)
        ui_file = QFile(path)
        ui_file.open(QFile.ReadOnly)
        ui  = loader.load(ui_file,parent)
        ui_file.close()
        return ui
    def __initPramsStampClass(self,data_cls=None):
        self.__ui.listWidgetSimple.clear()
        self.__ui.listWidget_header.clear()
        if data_cls is None:
            path_plugin=self.__parent_object.plugin_data_folder
            file_ext = "*.jstamp"
            last_files = getLastModifiFiles(path_plugin, file_ext)
            self.__cls_params=factoryParamsStamp(last_files)
        else:
            self.__cls_params=data_cls
        for key in self.__cls_params.keys():
            obj_params=self.__cls_params.get(key)
            if type(obj_params).__name__=="objSimple" or type(obj_params).__name__=='objSimpleNoData':
                self.__addWidgetInList(key,obj_params)
            if type(obj_params).__name__=="objHeader":
                self.__addWidgetInHeader(key,obj_params)

    def __addWidgetInList(self,name,object):
        widget_curent=WidgetParamDate(name,object)
        item_1 = QListWidgetItem()
        item_1.setSizeHint(widget_curent.sizeHint())
        self.__ui.listWidgetSimple.addItem(item_1)
        self.__ui.listWidgetSimple.setItemWidget(item_1, widget_curent)
    def __addWidgetInHeader(self,name,object):
        widget_curent=WidgetParamHeader(name,object)
        item_1 = QListWidgetItem()
        item_1.setSizeHint(widget_curent.sizeHint())
        self.__ui.listWidget_header.addItem(item_1)
        self.__ui.listWidget_header.setItemWidget(item_1, widget_curent)
    def __apply_cls_value(self):
        count_obj_simple=self.__ui.listWidgetSimple.count()
        for i in range(count_obj_simple):
            item_curent=self.__ui.listWidgetSimple.item(i)
            widget = self.__ui.listWidgetSimple.itemWidget(item_curent)
            cls_data_simple=widget.getUpdatedObject()
            self.__cls_params[widget.name_object]=cls_data_simple
        count_obj_header=self.__ui.listWidget_header.count()
        for i in range(count_obj_header):
            item_curent=self.__ui.listWidget_header.item(i)
            widget = self.__ui.listWidget_header.itemWidget(item_curent)
            cls_data_simple=widget.getUpdatedObject()
            self.__cls_params[widget.name_object]=cls_data_simple

    def __save(self):
        title = 'Сохранить настройки параметров штампа'
        filter = 'Параметры штампа (*.jstamp)'
        folder = self.__parent_object.plugin_data_folder
        selected_filter = None
        selected_filter = 'Параметры штампа (*.jstamp)'
        file_name='*.jstamp'
        file_name=None
        out_path = axipy.save_file_dialog(filter, title, folder,file_name,selected_filter)
        if out_path is None:
            return
        self.__apply_cls_value()
        saveObjectToJson(self.__cls_params,out_path)
    def __load(self):
        title = 'Сохранить настройки параметров штампа'
        filter = 'Параметры штампа (*.jstamp)'
        folder = self.__parent_object.plugin_data_folder
        str_folder=str(folder)+"\\"
        selected_filter = '*.jstamp'
        path_cls = axipy.open_file_dialog(filter, title, str_folder, selected_filter)
        if path_cls is None:
            return
        data_cls=None
        try:
            data_cls=factoryParamsStamp(path_cls)
        except:
            print("Ошибка чтения файла :"+str(path_cls))
        if data_cls is None:
            return
        self.__initPramsStampClass(data_cls)
    def getParametrsStamp(self):
        return self.__cls_params

