import glob
import os
from pathlib import Path

import axipy
from PySide2.QtCore import QObject, QSizeF
from PySide2.QtGui import QPageLayout, QPageSize
from axipy import data_manager, Table, Rectangle, LinearUnits

from ru_axioma_gis_stamp_to_report.ui.tools.pages_report.helper_printer import PageInfo


def pathTabIsOpen(path_table):
    temp_path=path_table
    if isinstance(path_table,str):
        temp_path=Path(path_table)
    for tab in data_manager.tables:
        file_name=tab.file_name
        if file_name is None:
            continue
        path_tab=Path(file_name)
        if path_tab==temp_path:
            return tab
    return None
def removeTable(path_tab):
    try:

        temp_path = path_tab
        if isinstance(path_tab, str):
            temp_path = Path(path_tab)
        if not temp_path.exists():
            return True
        axipy.provider_manager.tab.remove_table_files(temp_path)
        return True
    except Exception as Ex:
        return False
def getLastModifiFiles(path_folder,file_mask="*.*"):
    path_file=None
    if isinstance(path_folder,str):
        path_file=Path(path_folder).joinpath(file_mask)
    else:
        path_file=path_folder.joinpath(file_mask)
    files= glob.glob(str(path_file))
    if len(files)==0:
        return None
    files.sort(key=os.path.getmtime, reverse=True)
    return files[0]

class TablesInfo(QObject):
    def __init__(self):
        self.__container={}
        data_manager.removed.connect(self.__removed_table)
    def __removed_table(self,name_table):
        try:
            del self.__container[name_table]
        except:
            pass
    def __add_table(self,name_table):
        pass

class TableListInfo:
    __dictionary_tab_list={}
    def __init__(self):
        pass
    def addTable(self,table):

        tab_curent=table
        if isinstance(table,str):
            tab_curent=data_manager.find(table)
        if tab_curent.name in self.__dictionary_tab_list.keys():
            return

        features=list(tab_curent.items())
        if len(features)==0:
            pass
        else:
            page_info=self.__pageInfo(features[0].geometry,features[0]['scale'])
        self.__dictionary_tab_list[tab_curent.name]=page_info
    def removedTable(self,table):
        if isinstance(table,str):
            try:
                del self.__container[table]
            except:
                pass
            return
        tab_curent = data_manager.find(table)
        if tab_curent is None:
            return
        try:
            del self.__container[tab_curent.name]
        except:
            pass
        return

    def getPageinfo(self,name_tab):
        try:
            return self.__dictionary_tab_list[name_tab]
        except:
            return None
    def __pageInfo(self,geometry,scale)->PageInfo:
        if not isinstance(geometry,Rectangle):
            return None
        unit=geometry.coordsystem.unit
        width=geometry.xmax-geometry.xmin
        height=geometry.ymax-geometry.ymin
        width_meters=unit.to_unit(LinearUnits.m,width)
        height_meters=unit.to_unit(LinearUnits.m,height)
        width_sm=width_meters/scale
        height_sm=height_meters/scale
        orientation_page=QPageLayout.Portrait
        page_size=None
        if width_meters>height_meters:
            orientation_page = QPageLayout.Landscape
            page_size=self.__findPageSize(height_sm,width_sm)
        else:
            page_size = self.__findPageSize(width_sm,height_sm )
        return PageInfo(page_size,orientation_page)
    def __findPageSize(self,width_list,height_list):
        list_page=[QPageSize.A4,QPageSize.A3,QPageSize.A2,QPageSize.A1,QPageSize.A0]
        for page in list_page:
            width=QPageSize.size(page,QPageSize.Millimeter).width()/10
            height=QPageSize.size(page,QPageSize.Millimeter).height()/10
            if width_list<width and height_list<height:
                return page
        return None




