import math
import traceback

import axipy
from PySide2.QtCore import QObject, Signal
from axipy import LinearUnits, Rectangle, UnitValue, ListLayers, VectorLayer, Schema, Attribute, provider_manager, \
    Feature


class CalcIntProcent:
    def __init__(self,count):
        self.__last_proc=-1
        self.__count=count
    def getValue(self,curent):
        cur_val=int((curent/self.__count)*100+0.5)
        if cur_val==self.__last_proc:
            return None
        self.__last_proc=cur_val
        return cur_val

class CalculatorListFromBound(QObject):
    change_progress=Signal(int)
    change_progress_bar_format=Signal(str)
    def __init__(self,bound,view_map,width_list: axipy.UnitValue,height_list: axipy.UnitValue,scale:axipy.UnitValue,path_out_table):
        '''

        :param bound: область для формирования отчетов
        :param view_map: активная карта для которой будет формироваться много страничный отчет
        :param width_list: ширина листа карты в отчете
        :param height_list: высота листа карты в отчете
        :param scale : масштаб
        '''
        super().__init__()
        self.__source_bound=bound
        self.__active_map_view=view_map
        self.__width_out=width_list
        self.__height_out=height_list
        self.__scale=scale
        self.__createBound()
        self.__listBound=None
        self.__path_out_tab=path_out_table
    def __createBound(self):
        if self.__source_bound is None:
            self.__source_bound=Rectangle(self.__active_map_view.scene_rect,cs=self.__active_map_view.coordsystem)
    def run(self,onlyContainObj=False):
        try:
            self.__listBound=self.getListMap(onlyContainObj)
            self.__saveToTab()
        except Exception as ex:
            traceback.print_exc()
        return
    @property
    def listBounds(self):
        return self.__listBound
    def getListMap(self,onlyContainObj=False):
        print("Start run")
        cls_contain_obj=None
        if onlyContainObj:
            cls_contain_obj=MapViewDataSource(self.__active_map_view)
        if self.__width_out.unit!=LinearUnits.cm:

            width=self.__width_out.unit.to_unit(LinearUnits.cm,self.__width_out.value)
            heght=self.__height_out.unit.to_unit(LinearUnits.cm,self.__height_out.value)
        else:
            width = self.__width_out.value
            heght = self.__height_out.value
        cs=self.__source_bound.coordsystem
        unit_bounds=cs.unit
        rect_bound=self.__source_bound.bounds


        width_map=unit_bounds.to_unit(LinearUnits.m,rect_bound.width)
        map_center_x=unit_bounds.to_unit(LinearUnits.m,rect_bound.center.x)
        map_center_y = unit_bounds.to_unit(LinearUnits.m, rect_bound.center.y)
        height_map=unit_bounds.to_unit(LinearUnits.m,rect_bound.height)
        width_map_list=width*self.__scale.value
        height_map_list=heght*self.__scale.value
        count_list_w=int(math.ceil(width_map/width_map_list))
        count_list_h=int(math.ceil(height_map/height_map_list))
        lists_map=[]
        xmim=rect_bound.xmin
        ymax=rect_bound.ymax
        ''' относительно центра по y'''
        if count_list_h%2==0:
            ymax=map_center_y+height_map_list*count_list_h/2
        else:
            ymax=map_center_y+height_map_list/2+height_map_list*int(count_list_h/2)
        print("Count h:"+str(count_list_h))
        print("Count w:" + str(count_list_w))
        self.change_progress_bar_format.emit("Формирование листов разбивки")
        self.change_progress.emit(0)
        cls_procent=CalcIntProcent(count_list_h*count_list_w)
        common_index=0
        for i in range(count_list_h):
            cur_ymax=ymax-i*height_map_list
            for j in range(count_list_w):
                common_index=common_index+1
                cur_procent=cls_procent.getValue(common_index)
                if cur_procent is not None:
                    self.change_progress.emit(cur_procent)
                curent_xmin=xmim+j*width_map_list
                rect_map_list=Rectangle(curent_xmin,cur_ymax-height_map_list,curent_xmin+width_map_list,cur_ymax,cs=cs)
                if cls_contain_obj is not None:
                    isContain=cls_contain_obj.existDataInBound(rect_map_list)
                    if not isContain:
                        continue
                lists_map.append(rect_map_list)
        self.change_progress.emit(100)
        return lists_map
    def __saveToTab(self):
        print("Start save to table")
        schema = Schema(
            Attribute.integer('id_list'),
            Attribute.integer('scale'),
            coordsystem=self.__active_map_view.coordsystem
        )
        tab = provider_manager.tab.create_open(self.__path_out_tab, schema)
        print(tab)
        self.change_progress_bar_format.emit("Запись в таблицу")
        self.change_progress.emit(0)
        cls_procent = CalcIntProcent(len(self.__listBound))
        for i, bnd in enumerate(self.__listBound):
            cur_procent = cls_procent.getValue(i+1)
            if cur_procent is not None:
                self.change_progress.emit(cur_procent)
            ft = Feature({'id_list': i + 1,'scale':self.__scale.value}, geometry=bnd)
            tab.insert(ft)
def FindAllDataSource(layers):
    data_objects=[]
    for lyr in layers:
        #print("Source layer:"+lyr.title)
        if isinstance(lyr, ListLayers):
            res_comp= FindAllDataSource(lyr)
            if res_comp is not None:
                data_objects.extend(lyr)
            continue
        if isinstance(lyr,VectorLayer):
            data_objects.append(lyr.data_object)
    return data_objects
def containInObject(data_source,geo_obj):
    temp_geo=geo_obj
    if data_source.coordsystem!=geo_obj.coordsystem:
        temp_geo=geo_obj.reproject(data_source.coordsystem)

    for ft in data_source.itemsInObject(temp_geo):
        return True
    return False
class MapViewDataSource:
    def __init__(self,map_view):
        self.__map=map_view.map
        self.__data_source_in_map=FindAllDataSource(self.__map.layers)
    def existDataInBound(self,geo_obj):
        for source in self.__data_source_in_map:
            if containInObject(source,geo_obj):
                return True
        return False





