from pathlib import Path

from PySide2.QtGui import QPageSize, QPageLayout
from axipy import LinearUnits, UnitValue, Rect, Rectangle, Line, Pnt, Style, MapReportItem

#from ru_axioma_gis_stamp_to_report.Test.Test2.CreateReportAndStampAndMapBig import mapItem
from ru_axioma_gis_stamp_to_report.ui.tools.TransformAxi import addItemToReport
from ru_axioma_gis_stamp_to_report.ui.tools.db.dbTools import DbTools
from ru_axioma_gis_stamp_to_report.ui.tools.stamp_cls import loadStamp, loadStampDescription

def rotateRect(rect,angle,offset):
    temp_grect = Rectangle(rect).shift(offset,0)

    if angle==0:
        return temp_grect.bounds

    rect_rot=temp_grect.rotate((rect.xmin,rect.ymin),angle).bounds
    return rect_rot
def findParametr(list_params_stamp,name):
    for i,pr in enumerate(list_params_stamp):
        if pr.name==name:
            return i
    return None
class ParametricValue:
    def __init__(self,id,name,alias,name_date_alias=None,value=None):
        self.id=id
        self.name=name
        self.alias=alias
        self.date_alias=name_date_alias
        self.value=value
class PageForStamp:
    def __init__(self,db:DbTools):
        self.__db=db
        self.__def_offset=UnitValue(5, LinearUnits.mm)
        self.__page_size=None
        self.__orientation=0
        self.__dxl=5
        self.__dxr=5
        self.__dyup=5
        self.__dydown=5
        self.__unit_dxl = UnitValue(self.__dxl,LinearUnits.mm)
        self.__unit_dxr = UnitValue(self.__dxr,LinearUnits.mm)
        self.__unit_dyup =  UnitValue(self.__dyup,LinearUnits.mm)
        self.__unit_dydown = UnitValue(self.__dydown,LinearUnits.mm)
    def __initPageSize(self):
        s=QPageSize.size(self.__page_size, QPageSize.Millimeter)
        if self.__orientation==0:
            self.__width=UnitValue(s.width(),LinearUnits.mm)
            self.__height=UnitValue(s.height(),LinearUnits.mm)
        else:
            self.__width = UnitValue(s.height(), LinearUnits.mm)
            self.__height = UnitValue(s.width(), LinearUnits.mm)
    def load(self,id_obj:list,params_stamp):
        self.__loadParametrciValue(params_stamp)
        self.__data_stamps=[]
        for stamp_id in id_obj:
            cls_stamp=loadStampDescription(self.__db,stamp_id)

            cls_stamp.setParams(self.__list_parametric_values)
            self.__data_stamps.append(cls_stamp)
        self.__prepareBounds()

    def __loadParametrciValue(self,params_stamp):
        self.__list_parametric_values=[]
        sql="select * from prams_object_text"
        rows=self.__db.select(sql)
        for row in rows:
            self.__list_parametric_values.append(ParametricValue(row[0],row[1],row[2],row[3]))
        for params in self.__list_parametric_values:
            name = params.name

            value = None
            try:
                cls_param=params_stamp.get(name)
                value = str(cls_param.value)
                params.value = value
                if cls_param.date_create is not None:
                    index_param_date=findParametr(self.__list_parametric_values,params.date_alias)
                    if index_param_date is not None:
                        self.__list_parametric_values[index_param_date].value=str(cls_param.date_create.value)

            except:
                pass
            #params.value = value
        return
    def addParamList(self,name,value):
        index_param_date=findParametr(self.__list_parametric_values,name)
        if index_param_date is not None:
            self.__list_parametric_values[index_param_date].value = value

    def setPageSize(self,size_page,orientation=0):
        #if isinstance(size_page,QPageSize):
        self.__page_size=size_page
        self.__orientation=orientation
        self.__initPageSize()
        self.__prepareBounds()

    def __prepareBounds(self):
        if self.__page_size is None:
            return
        self.__main_page=Rect(self.__dxl,self.__dyup,self.__width.value-self.__dxr,self.__height.value-self.__dydown)
        ''' left size'''
        self.__bound_left=None
        self.__bound_right=None
        for stamp in self.__data_stamps:
            if stamp.position!=1:
                if self.__bound_right is None:
                    self.__bound_right=stamp.bound
                else:
                    self.__bound_right=self.__bound_right.merger(stamp.bound)
                continue
            stamp_rect=rotateRect(stamp.bound,stamp.angle,stamp.offset)
            if self.__bound_left is None:
                self.__bound_left=stamp_rect
            else:
                self.__bound_left=self.__bound_left.merge(stamp_rect)
        dx_left=self.__bound_left.width
        dy_bottom=self.__bound_right.height
        self.__map_bound=Rect(self.__main_page.xmin+dx_left,self.__main_page.ymin,self.__main_page.xmax,self.__main_page.ymax-dy_bottom)


    def mapBound(self,number_vertical_page,unit_bound=LinearUnits.cm):
        temp_rect=Rect(self.__map_bound.xmin,self.__map_bound.ymin,self.__map_bound.xmax,self.__map_bound.ymax)
        height=temp_rect.height
        temp_rect.ymin=temp_rect.ymin+number_vertical_page*self.__height.value
        temp_rect.ymax = temp_rect.ymin + height
        if unit_bound==LinearUnits.mm:

            return temp_rect
        coef=LinearUnits.mm.to_unit(unit_bound)
        return Rect(temp_rect.xmin*coef,temp_rect.ymin*coef,temp_rect.xmax*coef,temp_rect.ymax*coef)
    def boundWork(self,number_vertical_page,unit_bound=LinearUnits.cm):
        temp_rect=Rect(self.__main_page.xmin,self.__main_page.ymin,self.__main_page.xmax,self.__main_page.ymax)
        height = temp_rect.height
        temp_rect.ymin = temp_rect.ymin + number_vertical_page * self.__height.value
        temp_rect.ymax = temp_rect.ymin + height
        if unit_bound == LinearUnits.mm:
            return temp_rect
        coef = LinearUnits.mm.to_unit(unit_bound)
        return Rect(self._PageForStamp__map_bound.xmin * coef, temp_rect.ymin * coef, temp_rect.xmax * coef, temp_rect.ymax * coef)


    def drawBaseStamp(self,view_report,number_page_vertical=0):
        report = view_report.report
        page_size = report.page_size
        cls_stamp_2=self.__data_stamps[0]
        unit = report.unit
        bound_stamp_2 = cls_stamp_2.bound

        unit_stamp = cls_stamp_2.unit
        scale = unit_stamp.to_unit(unit)
        start_x = bound_stamp_2.height * scale
        #tart_y = page_size.height() - bound_stamp_2.width * scale
        # start_y=page_size.height()

        start_x = bound_stamp_2.height * scale+self.__getValueToReportUnits(self.__unit_dxl,unit)
        # start_y=page_size.height()-bound_stamp.height/10
        # start_y=page_size.height()-bound_stamp.width*scale
        start_y = page_size.height()*(number_page_vertical+1)-self.__getValueToReportUnits(self.__unit_dydown,unit)
        cls_stamp_2.paramsTransform(start_x, start_y, -90, unit)
        trasfoms_objects = cls_stamp_2.getTransfromGeometry(True)
        for obj in trasfoms_objects:
            addItemToReport(report, obj['geometry'], obj['style'])
        cls_stamp = self.__data_stamps[1]
        bound_stamp = cls_stamp.bound
        unit_stamp = cls_stamp.unit
        scale = unit_stamp.to_unit(unit)
        start_x = page_size.width() - bound_stamp.width * scale-self.__getValueToReportUnits(self.__unit_dxr,unit)
        # start_y=page_size.height()-bound_stamp.height/10
        #start_y = page_size.height() - bound_stamp.width * scale
        start_y = page_size.height()*(number_page_vertical+1)-self.__getValueToReportUnits(self.__unit_dydown,unit)
        cls_stamp.paramsTransform(start_x, start_y, 0, unit)
        trasfoms_objects = cls_stamp.getTransfromGeometry(True)
        for obj in trasfoms_objects:
            addItemToReport(report, obj['geometry'], obj['style'])
        self.__drawSimpleRamka(report,number_page_vertical)
    def drawBigStamp(self,view_report,number_page_vertical=0):
        report = view_report.report
        page_size = report.page_size
        unit = report.unit
        cls_stamp_3 = self.__data_stamps[2]
        cls_stamp_2 = self.__data_stamps[1]
        bound_stamp_2 = cls_stamp_2.bound
        bound_stamp_3 = cls_stamp_3.bound
        unit_stamp = cls_stamp_3.unit
        scale = unit_stamp.to_unit(unit)
        start_x = bound_stamp_3.height * scale+self.__getValueToReportUnits(self.__unit_dxl,unit)
        # start_y=page_size.height()-bound_stamp.height/10
        start_y = page_size.height() *(number_page_vertical+1)- bound_stamp_2.width * scale-self.__getValueToReportUnits(self.__unit_dydown,unit)
        # start_y=page_size.height()
        cls_stamp_3.paramsTransform(start_x, start_y, -90, unit)
        trasfoms_objects = cls_stamp_3.getTransfromGeometry(True)
        for obj in trasfoms_objects:
            addItemToReport(report, obj['geometry'], obj['style'])
        start_x = bound_stamp_3.height * scale+self.__getValueToReportUnits(self.__unit_dxl,unit)
        # start_y=page_size.height()-bound_stamp.height/10
        # start_y=page_size.height()-bound_stamp.width*scale
        start_y = page_size.height()*(number_page_vertical+1)-self.__getValueToReportUnits(self.__unit_dydown,unit)
        cls_stamp_2.paramsTransform(start_x, start_y, -90, unit)
        trasfoms_objects = cls_stamp_2.getTransfromGeometry(True)
        for obj in trasfoms_objects:
            addItemToReport(report, obj['geometry'], obj['style'])
        cls_stamp = self.__data_stamps[0]
        bound_stamp = cls_stamp.bound
        unit_stamp = cls_stamp.unit
        scale = unit_stamp.to_unit(unit)
        start_x = page_size.width() - bound_stamp.width * scale-self.__getValueToReportUnits(self.__unit_dxl,unit)


        start_y = page_size.height()*(number_page_vertical+1)-self.__getValueToReportUnits(self.__unit_dydown,unit)
        cls_stamp.paramsTransform(start_x, start_y, 0, unit)
        trasfoms_objects = cls_stamp.getTransfromGeometry(True)
        for obj in trasfoms_objects:
            addItemToReport(report, obj['geometry'], obj['style'])
        self.__drawSimpleBigRamka(report,number_page_vertical)

    def __getValueToReportUnits(self,value_in_units,unit_report):
        if value_in_units.unit==unit_report:
            return value_in_units.value
        new_value=value_in_units.unit.to_unit(unit_report,value_in_units.value)
        return new_value
    def __drawSimpleRamka(self,report,number_page_vertical=0):
        cls_stamp_2 = self.__data_stamps[0]
        unit = report.unit
        page_size = report.page_size
        bound_stamp_2 = cls_stamp_2.bound

        unit_stamp = cls_stamp_2.unit
        scale = unit_stamp.to_unit(unit)
        x_min=bound_stamp_2.height*scale+self.__getValueToReportUnits(self.__unit_dxl,unit)
        x_max=page_size.width()-self.__getValueToReportUnits(self.__unit_dxr,unit)
        y=self.__getValueToReportUnits(self.__unit_dyup,unit)+(number_page_vertical*page_size.height())
        line = Line(Pnt(x_min, y), Pnt(x_max, y))
        style = Style.from_mapinfo('Pen (25, 2, 0)')
        addItemToReport(report, line, style)
        y_max=page_size.height()*(number_page_vertical+1)-self.__getValueToReportUnits(self.__unit_dydown,unit)
        line = Line(Pnt(x_max, y), Pnt(x_max, y_max))
        addItemToReport(report, line, style)
        line = Line(Pnt(x_min, y), Pnt(x_min, y_max))
        addItemToReport(report, line, style)

    def __drawSimpleBigRamka(self,report,number_page_vertical=0):
        cls_stamp_2 = self.__data_stamps[2]
        unit = report.unit
        page_size = report.page_size
        bound_stamp_2 = cls_stamp_2.bound

        unit_stamp = cls_stamp_2.unit
        scale = unit_stamp.to_unit(unit)
        x_min=bound_stamp_2.height*scale+self.__getValueToReportUnits(self.__unit_dxl,unit)
        x_max=page_size.width()-self.__getValueToReportUnits(self.__unit_dxr,unit)
        y=self.__getValueToReportUnits(self.__unit_dyup,unit)+(number_page_vertical*page_size.height())
        line = Line(Pnt(x_min, y), Pnt(x_max, y))
        style = Style.from_mapinfo('Pen (25, 2, 0)')
        addItemToReport(report, line, style)
        y_max=page_size.height()*(number_page_vertical+1)-self.__getValueToReportUnits(self.__unit_dydown,unit)
        line = Line(Pnt(x_max, y), Pnt(x_max, y_max))
        addItemToReport(report, line, style)
        line = Line(Pnt(x_min, y), Pnt(x_min, y_max))
        addItemToReport(report, line, style)
class DbStampPage:
    __default_db='v1_stamp.db'
    def __init__(self,param_stamp=None,path_db=None):
        self.__pageBig=None
        self.__pageStandart=None
        self.__param_stamp=param_stamp
        self.__load()
    def __load(self,path_db=None):
        db_path=path_db
        if path_db is None:
            db_path=Path(__file__).parent.joinpath(self.__default_db)
        db=DbTools(str(db_path))
        self.__pageBig=PageForStamp(db)
        self.__pageStandart = PageForStamp(db)
        self.__pageBig.load([1,2,3],self.__param_stamp)
        self.__pageStandart.load([4,5],self.__param_stamp)
        db.close()
    @property
    def pageBigStamp(self):
        return self.__pageBig
    @property
    def pageStandartStamp(self):
        return self.__pageStandart
    def getBoundMapBigPage(self,size_page,orientation,unit=LinearUnits.cm):
        int_oriantation_page=0
        if orientation==QPageLayout.Landscape:
            int_oriantation_page=1
        self.__pageBig.setPageSize(size_page,int_oriantation_page)
        return self.__pageBig.mapBound(0,unit)
    def getBoundMapStandartPage(self,size_page,orientation,unit=LinearUnits.cm):
        int_oriantation_page=0
        if orientation==QPageLayout.Landscape:
            int_oriantation_page=1
        self.__pageStandart.setPageSize(size_page,int_oriantation_page)
        return self.__pageStandart.mapBound(0,unit)
    def getWorkBoundPage(self,size_page,orientation,unit=LinearUnits.cm):
        int_oriantation_page = 0
        if orientation == QPageLayout.Landscape:
            int_oriantation_page = 1
        self.__pageBig.setPageSize(size_page, int_oriantation_page)
        return self.__pageBig.boundWork(0, unit)
    def getCurentWorkBound(self,number_page,page_info,unit=LinearUnits.cm):
        int_oriantation_page = 0
        if page_info.orientation == QPageLayout.Landscape:
            int_oriantation_page = 1
        self.__pageBig.setPageSize(page_info.page_size, int_oriantation_page)
        return self.__pageBig.boundWork(number_page, unit)
    def printBigStamp(self,viewreport,page_number=0):
        self.__pageBig.drawBigStamp(viewreport,page_number)
    def printStandartStamp(self,viewreport,page_number=0):
        self.__pageStandart.drawBaseStamp(viewreport,page_number)
class MapPrinter:
    def __init__(self,reportview,mapview,pages,page_info):
        self.__reportview=reportview
        self.__mapview=mapview
        self.__pages=pages
        self.__page_info=page_info
    def printToReport(self,geometry,scale,page_number):
        self.__reportview.report.unit=LinearUnits.cm
        rect_geometry=geometry.bounds
        width_rect=rect_geometry.width
        height_rect=rect_geometry.height
        unit_geo=geometry.coordsystem.unit
        width_meters=unit_geo.to_unit(LinearUnits.m,width_rect)
        height_meters = unit_geo.to_unit(LinearUnits.m, height_rect)
        width_sm=width_meters/scale
        height_sm=height_meters/scale
        bound_work_report=self.__pages.getCurentWorkBound(page_number,self.__page_info)

        xmin_report=self.__getValueToReportUnits(UnitValue(bound_work_report.xmin,LinearUnits.cm))
        ymin_report=self.__getValueToReportUnits(UnitValue(bound_work_report.ymin,LinearUnits.cm))
        xmax_report=self.__getValueToReportUnits(UnitValue(xmin_report+width_sm,LinearUnits.cm))
        ymax_report=self.__getValueToReportUnits(UnitValue(ymin_report+height_sm,LinearUnits.cm))

        map_bound_report=Rect(xmin_report,ymin_report,xmax_report,ymax_report)
        mapItem=MapReportItem(map_bound_report,self.__mapview.map)
        pt_center=geometry.centroid()
        mapItem.center=Pnt(pt_center.x,pt_center.y)
        mapItem.scale=scale*100
        self.__reportview.report.items.add(mapItem)



    def __getValueToReportUnits(self,value_in_units):
        if value_in_units.unit==self.__reportview.report.unit:
            return value_in_units.value
        new_value=value_in_units.unit.to_unit(self.__reportview.report.unit,value_in_units.value)
        return new_value










