import math

import axipy
from PySide2.QtGui import QTransform
from axipy import Geometry, CoordSystem, Style, LinearUnit, LinearUnits, GeometryCollection, Text, Pnt, Point

from ru_axioma_gis_stamp_to_report.ui.tools.TransformAxi import AxiTransform, correctMifText
from ru_axioma_gis_stamp_to_report.ui.tools.db.dbTools import DbTools



def transformText(geo_text,scale,dx,dy):
    start_point=geo_text.startPoint
    end_point=geo_text.endPoint
    new_start_x=dx+start_point.x*scale
    new_start_y =dy+ start_point.y * scale
    geo_text.startPoint=Pnt(new_start_x,new_start_y)
    return geo_text
def transformMifText(mif_text,tr,angle=0):
    ind_start=mif_text.rfind('"')
    sub_bound=mif_text[ind_start+1:len(mif_text)].strip()
    xy_bound=sub_bound.split(' ')
    xmin=float(xy_bound[0])
    ymin=float(xy_bound[1])
    xmax=float(xy_bound[2])
    ymax=float(xy_bound[3])
    dy=ymax-ymin

    pt1=Point(xmin,ymin-dy)
    pt1_n=Point(xmin,ymax-dy)
    pt2 = Point(xmax,ymax-dy)
    pt2_n = Point(xmax, ymin - dy)
    pt_1_t = pt1_n.affine_transform(tr)
    #pt_1_t=pt1.affine_transform(tr)
    pt_2_t = pt2.affine_transform(tr)
    pt_2_t = pt2_n.affine_transform(tr)
    out_mif=mif_text[0:ind_start+1]+" "+str(pt_1_t.x)+" "+str(pt_1_t.y)+" "+str(pt_2_t.x)+" "+str(pt_2_t.y)
    if angle!=0:
        out_mif=out_mif+" Angle "+str(angle)
    return Geometry.from_mif(out_mif)
def findById(list_cls,id):
    for item in list_cls:
        if item.id==id:
            return item
    return None
class Stamp:
    def __init__(self):
        self.__geometrys_and_style=[]
        self.__bound=None
        self.__cs=None
        self.__unit=LinearUnits.mm
        self.angle=0
        self.offset=0
        self.position=0
        self.__list_parametrs_values=None
        '''
        position=0 правый нижний угол листа
        position=1 левый нижний угол листа
        '''
    def setParams(self,list_params):
        self.__list_parametrs_values=list_params
    @property
    def unit(self):
        return self.__unit
    def insert(self,geometry_mif,style,id):
        geo=Geometry.from_mif(geometry_mif)
        mi_style = Style.from_mapinfo(style)
        if not isinstance(geo,Text):


            geo_bound=geo.bounds
            if self.__bound is None:
                self.__bound=geo_bound
            else:
                self.__bound=self.__bound.merge(geo_bound)
        self.__geometrys_and_style.append({'id':id,'geometry':geo,'style':mi_style,'mif':geometry_mif})
    def paramsTransform(self,dx,dy,angle,unit,pt_rotate=None):
        scale=self.__unit.to_unit(unit)
        self.__axi_tarnsfrom=AxiTransform(dx,dy,angle,scale,pt_rotate)
    def __geometryTransfrom(self,geo):
        return self.__axi_tarnsfrom.transform(geo)
    def getTransfromGeometry(self,isCombine=False):
        out_object = []
        for geo_style in self.__geometrys_and_style:
            geo = geo_style['geometry']
            if isinstance(geo, Text):
                params=findById(self.__list_parametrs_values,geo_style['id'])
                geo_text=correctMifText(geo_style['mif'])
                if params is not None:
                    geo_text.text=params.value
                new_geo= self.__geometryTransfrom(geo_text)
                out_object.append({"geometry":new_geo,'style':geo_style['style']})
                continue
            new_geo = self.__geometryTransfrom(geo)
            out_object.append({"geometry": new_geo, 'style': geo_style['style']})
        if isCombine :
            geo_col=GeometryCollection()
            new_out_object=[]
            out_style=None
            for item in out_object:
                geo=item['geometry']
                if isinstance(geo,Text):
                    new_out_object.append(item)
                else:
                    geo_col.append(geo)
                    if out_style is None:
                        out_style=item['style']
            new_out_object.append({'geometry':geo_col,'style':out_style})
            return new_out_object
        return out_object

    def getGeometry(self,dx,dy,units,isCombine=False):
        scale=self.__unit.to_unit(units)

        self.__tr= QTransform.fromScale(scale, -scale)

        if self.angle != 0:
            angle_radians = math.radians(self.angle)
            cosa = math.cos(angle_radians)
            sina = math.sin(angle_radians)
            self.__tr = QTransform().rotate(float(self.angle))
            self.__tr *=QTransform.fromScale(scale,-scale)

        self.__tr *= QTransform.fromTranslate(dx, dy)
        out_object=[]
        #is_fisrt_text=True
        for geo_style in self.__geometrys_and_style:
            geo=geo_style['geometry']
            if isinstance(geo,Text):
                mif_text=geo.to_mif()
                #new_geo=transformMifText(mif_text,tr,self.angle)
                new_geo = self.__transformMifText(mif_text)
                '''
                if is_fisrt_text:
                    is_fisrt_text=False
                else:
                    continue
                '''
            else:
                new_geo=geo.affine_transform(self.__tr)
            out_object.append({"geometry":new_geo,'style':geo_style['style']})

        return out_object
    def __transformMifText(self,mif_text):
        ind_start = mif_text.rfind('"')
        sub_bound = mif_text[ind_start + 1:len(mif_text)].strip()
        xy_bound = sub_bound.split(' ')
        xmin = float(xy_bound[0])
        ymin = float(xy_bound[1])
        xmax = float(xy_bound[2])
        ymax = float(xy_bound[3])
        dy = ymax - ymin
        if self.angle==0:
            pt1 = Point(xmin, ymin - dy)
            pt2 = Point(xmax, ymax - dy)
            pt_1_t = pt1.affine_transform(self.__tr)
            pt_2_t = pt2.affine_transform(self.__tr)
            out_mif = mif_text[0:ind_start + 1] + " " + str(pt_1_t.x) + " " + str(pt_1_t.y) + " " + str(
                pt_2_t.x) + " " + str(pt_2_t.y)
            return Geometry.from_mif(out_mif)
        if self.angle==90:
            pt1 = Point(xmin, ymax - dy)
            pt2 = Point(xmax, ymin - dy)
            temp_mif=mif_text[0:ind_start + 1] + " " + str(pt1.x) + " " + str(pt1.y) + " " + str(pt2.x) + " " + str(pt2.y)
            temp_geo=Geometry.from_mif(temp_mif)
            new_text=temp_geo.affine_transform(self.__tr)
            #new_mif=new_text.to_mif()
            #new_text=Geometry.from_mif(new_mif)
            print("Width:"+str(new_text.width))
            print("Width:" + str(new_text.bounds.width))
            w_text=new_text.bounds.width
            new_text=new_text.shift(-2*w_text,0)
            return  new_text
            '''
            pt_1_t = pt1.affine_transform(self.__tr)
            pt_2_t = pt2.affine_transform(self.__tr)
            out_mif = mif_text[0:ind_start + 1] + " " + str(pt_1_t.x) + " " + str(pt_1_t.y) + " " + str(
                pt_2_t.x) + " " + str(pt_2_t.y)
            out_mif = out_mif + " Angle " + str(self.angle)
            

            return Geometry.from_mif(out_mif)
            '''

    @property
    def bound(self):
        return self.__bound

def loadStamp(db_obj,id_stamp,name_tab='stamps'):
    db=None
    if isinstance(db_obj,str):
        db=DbTools(db_obj)
    else:
        db=db_obj
    cls_stamp=Stamp()
    sql="select geometry_mif,mi_style,id from "+name_tab+" where id_stamp="+str(id_stamp)
    rows=db.select(sql)
    for row in rows:
        cls_stamp.insert(row[0],row[1],row[2])
    if isinstance(db_obj,str):
        db.close()
    return cls_stamp
def loadStampDescription(db_obj,id_stamp,tab_name='descriptions_stamps'):
    db = None
    if isinstance(db_obj, str):
        db = DbTools(db_obj)
    else:
        db = db_obj
    sql_description="Select angle,position,offset,id_stamp from "+tab_name+ ' where id='+str(id_stamp)
    rows = db.select(sql_description)
    if len(rows)==0:
        return None
    angel=rows[0][0]
    position=rows[0][1]
    offset=rows[0][2]
    id_stamp=rows[0][3]
    cls_stamp=loadStamp(db_obj,id_stamp)
    if cls_stamp is None:
        return None
    cls_stamp.offset=offset
    cls_stamp.angle=angel
    cls_stamp.position=position
    return cls_stamp








